@@ -1,9 +1,6 @@
 
 library(exactRankTests)
-actversion <- paste(R.version$major, R.version$minor, sep=".")
-thisversion <- "1.7.0"
-if (compareVersion(actversion, thisversion) >= 0)
-  RNGversion("1.6.2")
+RNGkind("Wichmann-Hill", "Kinderman-Ramage")
 set.seed(29081975)
 
 # test the distribution functions
